import numpy as np
from data_utils import process_data, seqnn_processing, f1, scorer
from keras.models import Sequential
from keras.layers import Embedding, Bidirectional, Activation, InputLayer, TimeDistributed, Dense, LSTM

"""
This script provides a simple architecture for sequential processing of sentences for
text classification. You can load a pretrained model and play around with it
or adjust things to train your own model. However this will most likely be
infeasible if you use one of the lab's computers so beware.

A sequential model achieves F-Scores for B and I tags of more than 0.9
"""


def eval(model):
    output = model.predict(test_X, verbose=1)
    predict_labels = np.argmax(output, axis=-1)
    gold_labels = np.argmax(test_y, axis=-1)
    flat_p = predict_labels.reshape(-1)
    flat_g = gold_labels.reshape(-1)
    scorer(flat_p, flat_g, label2index['B'], label2index['I'])
    return True


def build_model(input_size, vocab_size, tag_size):
    model = Sequential()
    model.add(InputLayer(input_shape=(max_len, )))
    model.add(Embedding(vocab_size, 128))
    model.add(Bidirectional(LSTM(256, return_sequences=True)))
    model.add(TimeDistributed(Dense(tag_size)))
    model.add(Activation('softmax'))

    model.compile(loss='categorical_crossentropy',
                  optimizer='adam',
                  metrics=[f1])
    return model


train_docs, test_docs, train_labels, test_labels, word2index, label2index = process_data('full_data.txt')
train_X, test_X, train_y, test_y = seqnn_processing(train_docs, test_docs,
                                                    train_labels, test_labels, word2index, label2index)

max_len = max(train_X.shape[1], test_X.shape[1])
vocab_size = len(word2index)
label_size = len(label2index)


model = build_model(max_len, vocab_size, label_size)
model.load_weights('seqnn.h5')
model.summary()

#model.fit(train_X, train_y, batch_size=16, epochs=10, validation_split=0.2)
#model.save_weights('seqnn.h5')

eval(model)
